#include <vmgp.h>
#include <vstream.h>
#include <datacertlib.h>
#include "app.h"
#include "globals.h"
#include "utility.h"
#include "res.h"
#include "unit.h"
#include "inventoryitem.h"

//Makes sure all the resources start out as NULL
void initResources(uint8_t* resources[], uint8_t numResources)
{
	uint8_t i;
	for (i = 0; i < numResources; i++)
		resources[i] = NULL;
}

//Frees all the resources from memory
void freeAllResources(uint8_t* resources[], uint8_t numResources)
{
	uint8_t i;
	for (i = 0; i < numResources; i++)
		if (resources[i] != NULL)
		{
			vDisposePtr( resources[i] );
			resources[i] = NULL;
		}
}

void getSpecs(uint16_t* width, uint16_t* height, uint8_t* vibrate)
{
	SYSCAPS syscaps;
	syscaps.size = sizeof(SYSCAPS);

	vGetCaps(CAPS_SYSTEM, (void*)&syscaps);
	*vibrate = (syscaps.flags & SYSTEM_VIBRATE);

	VIDEOCAPS videocaps;
	videocaps.size = sizeof(VIDEOCAPS);

	vGetCaps(CAPS_VIDEO, (void*)&videocaps);
	*width = videocaps.width;
	*height = videocaps.height;
	
}

void loadCollection()
{
	int32_t handle = vStreamOpen("collection.txt", STREAM_FILE | STREAM_READ | STREAM_BINARY);
	if (handle == -1)
	{
		uint8_t i;
		for (i = 0; i < NUM_TREASURES; i++)
			treasurecollection[i] = 0;
	}
	else
	{
		vStreamRead(handle, treasurecollection, NUM_TREASURES);
		vStreamClose(handle);
	}
}

void saveCollection()
{
	int32_t handle = vStreamOpen("collection.txt", STREAM_FILE | STREAM_CREATE | STREAM_WRITE | STREAM_TRUNC | STREAM_BINARY );
	vStreamWrite(handle, treasurecollection, NUM_TREASURES);
	vStreamClose(handle);
}

void saveGame()
{
	int32_t handle = vStreamOpen("save.txt", STREAM_FILE | STREAM_WRITE | STREAM_CREATE | STREAM_TRUNC | STREAM_BINARY );

	vStreamWrite(handle, seeds, NUM_LEVELS * sizeof(uint32_t));
	vStreamWrite(handle, sprites, MAX_SPRITES * sizeof(UNIT));
	vStreamWrite(handle, inventory, MAX_INVENTORY * sizeof(INVENTORYITEM));
	
	uint8_t buffer[1] = { level };
	vStreamWrite(handle, buffer, sizeof(uint8_t));

	vStreamClose(handle);
}

void loadGame(uint8_t *resources[])
{
	int32_t handle = vStreamOpen("save.txt", STREAM_FILE | STREAM_READ | STREAM_BINARY);

	if (handle != -1)
	{
		vStreamRead(handle, seeds, NUM_LEVELS * sizeof(uint32_t));
		vStreamRead(handle, sprites, MAX_SPRITES * sizeof(UNIT));
		vStreamRead(handle, inventory, MAX_INVENTORY * (sizeof(INVENTORYITEM)));
		
		uint8_t buffer[1];
		vStreamRead(handle, buffer, sizeof(uint8_t));
		level = buffer[0];

		vStreamClose(handle);

		int32_t h = vFileCreate("save.txt"); //Overwrite the contents of the save
		vStreamClose(h);

		uint8_t j;
		for (j = 0; j < MAX_SPRITES; j++)
		{
			if (resources[unitstats[j].id] == NULL)
				LoadPackedRes(unitstats[j].id, PELLET_UNPACKED_SIZE, &resources[unitstats[j].id]);
		}

	}
}

uint8_t saveGameExists()
{
	
	int32_t handle = vStreamOpen("save.txt", STREAM_READ);
	if (handle == -1) return 0;
	
	char buffer[10];
	int32_t ret = vStreamRead(handle, buffer, 10);
	if (ret == -1 || ret == 0) 
	{
		vStreamClose(handle);
		return 0;
	}

	vStreamClose(handle);
	
	return 1;
}

void saveSettings()
{
	int32_t handle = vStreamOpen("settings.txt", STREAM_FILE | STREAM_WRITE | STREAM_CREATE | STREAM_TRUNC | STREAM_BINARY );
	uint8_t buffer[2];
	buffer[0] = sound;
	buffer[1] = doesVibrate;
	vStreamWrite(handle, buffer, 2 );
	vStreamClose(handle);
}

void loadSettings()
{
	int32_t handle = vStreamOpen("settings.txt", STREAM_FILE | STREAM_READ | STREAM_BINARY );
	if (handle != -1)
	{
		uint8_t buffer[2];
		vStreamRead(handle, buffer, 2);
		sound = buffer[0];
		doesVibrate = buffer[1];
		vStreamClose(handle);
	}
}

void displayMessage(char *message, uint8_t* resources[])
{
	vFlipScreen(1);
	vDrawObject(0, height - TEXTBOX_HEIGHT, resources[TEXTBOX]);
	printWordWrap(	message, TEXTBOX_BORDER, height - TEXTBOX_HEIGHT + TEXTBOX_BORDER, 
					TEXTBOX_WIDTH - TEXTBOX_BORDER, THEFONT_WIDTH, THEFONT_HEIGHT);
	vFlipScreen(1);
	sleep(500);

	waitForKeyPress();
}

void getValidLocation(uint8_t* map, uint8_t* retX, uint8_t* retY)
{
	while(1)
	{
		uint8_t x = vGetRandom() % MAP_SIZE_X;
		uint8_t y = vGetRandom() % MAP_SIZE_Y;

		if ( map[y*MAP_SIZE_X+x] == 2 )
		{
			//We're on a valid tile.  Now make sure we're not overlapping any sprites
			uint8_t i, isValid = 1;
			for (i = 0; i < MAX_SPRITES; i++)
				if (sprites[i].isAlive && sprites[i].x == x * TILESIZE && sprites[i].y == y * TILESIZE)
					isValid = 0;

			if (isValid)
			{
				*retX = x;
				*retY = y;
				break;
			}
		}
	}
}

void revealTrap(uint8_t tx, uint8_t ty, uint8_t* resources[], uint8_t perm)
{
	//Handle what happens if the player moves over a trap
	if (map[ty * MAP_SIZE_X + tx] == TRAP_POISON)
	{

		if (perm) map[ty * MAP_SIZE_X + tx] = TRAP_POISON+perm;
		
		if (sprites[0].x / TILESIZE == tx && sprites[0].y / TILESIZE == ty)
		{
			sprites[0].statusTimePoison = POISON_LENGTH;
			flash(vRGB(0,255,0), FLASH_LENGTH);
			displayMessage("YOU ARE SPRAYED BY A POISON GAS FROM A TRAP", resources);
		}
		
		setActualTile2(tx*2, ty*2, 69, 70, 79, 80);
	}
	if (map[ty * MAP_SIZE_X + tx] == TRAP_CONFUSION)
	{
		if (perm) map[ty * MAP_SIZE_X + tx] = TRAP_CONFUSION+perm;

		if (sprites[0].x / TILESIZE == tx && sprites[0].y / TILESIZE == ty)
		{
			sprites[0].statusTimeConfusion = CONFUSION_LENGTH;
			flash(vRGB(255,255,0), FLASH_LENGTH);
			displayMessage("A YELLOW GAS ARISES FROM THE TILE MAKING YOU FEEL QUITE DIZZY", resources);
		}		
		setActualTile2(tx*2, ty*2, 61, 62, 71, 72);
	}
	if (map[ty * MAP_SIZE_X + tx] == TRAP_HEALTH)
	{
		if (perm) map[ty * MAP_SIZE_X + tx] = TRAP_HEALTH+perm;

		if (sprites[0].x / TILESIZE == tx && sprites[0].y / TILESIZE == ty)
		{
			sprites[0].hp = 100;
			flash(vRGB(255,255,255), FLASH_LENGTH);
			displayMessage("YOU REGAINED ALL YOUR HEALTH", resources);
		}

		setActualTile2(tx*2, ty*2, 25, 26, 35, 36);
	}
	if (map[ty * MAP_SIZE_X + tx] == TRAP_BOMB)
	{
		if (perm) map[ty * MAP_SIZE_X + tx] = TRAP_BOMB+perm;

		if (sprites[0].x / TILESIZE == tx && sprites[0].y / TILESIZE == ty)
		{
			sprites[0].hp -= 25;
			flash(vRGB(255,0,0), FLASH_LENGTH);
			displayMessage("A LARGE EXPLOSION ARISES FROM THE TRAP ENGULFING YOU IN FLAMES", resources);
		}

		setActualTile2(tx*2, ty*2, 29, 30, 39, 40);
	}
}

uint8_t prompt(char* message, uint8_t* resources[])
{
	vFlipScreen(1);
	vDrawObject(0, height - TEXTBOX_HEIGHT, resources[TEXTBOX]);
	
	printWordWrap(	message, TEXTBOX_BORDER, height - TEXTBOX_HEIGHT + TEXTBOX_BORDER, 
					width - TEXTBOX_BORDER, THEFONT_WIDTH, THEFONT_HEIGHT);

	printWordWrap( "1.YES  2.NO", TEXTBOX_BORDER, height-TEXTBOX_BORDER-THEFONT_HEIGHT, width - TEXTBOX_BORDER, THEFONT_WIDTH, THEFONT_HEIGHT);

	vFlipScreen(1);
	sleep(500);

	while(1)
	{
		if (keyPress('1')) return 1;
		if (keyPress('2')) return 2;
	}
}


